//+------------------------------------------------------------------+
//|                                              stochastic momentum |
//|                                                           mladen |
//|                                                                  |
//| Stochastic momentum developed by William Blau                    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  Red
#property indicator_color2  Goldenrod
#property indicator_color3  Lime
#property indicator_color4  Red
#property indicator_minimum -100
#property indicator_maximum  100
#property indicator_level1   0
#property indicator_level2   50
#property indicator_level3  -50
#property indicator_levelcolor DarkSlateGray


//
//
//
//
//

extern string TimeFrame   = "current time frame";
extern int    Length      = 13;
extern int    Smooth1     = 25;
extern int    Smooth2     =  2;
extern int    Signal      =  5;
extern int    Price       = PRICE_CLOSE;
extern bool   Delayed     = false;
extern bool   ShowArrows  = true;
extern bool   Interpolate = true;

//
//
//
//
//

double smo[];
double sms[];
double aup[];
double ado[];
double state[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,smo); SetIndexLabel(0,"Stochastic momentum");
   SetIndexBuffer(1,sms); SetIndexLabel(1,"Stochastic momentum signal");
   SetIndexBuffer(2,aup); 
   SetIndexBuffer(3,ado); 
   SetIndexBuffer(4,state);
      if (ShowArrows)
      {
         SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,108);
         SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,108);
      }
      else
      {
         SetIndexStyle(2,DRAW_NONE);
         SetIndexStyle(3,DRAW_NONE);
      }

   //
   //
   //
   //
   //
   
         Length  = MathMax(Length ,1);
         Smooth1 = MathMax(Smooth1,1);
         Smooth2 = MathMax(Smooth2,1);

      //
      //
      //
      //
      //
         
         IndicatorFileName = WindowExpertName();
         CalculateValue    = (TimeFrame=="calculateSmi"); if (CalculateValue) return(0);
         ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //
   
   string PriceType;
      switch(Price)
      {
         case PRICE_CLOSE:    PriceType = "Close";    break;  // 0
         case PRICE_OPEN:     PriceType = "Open";     break;  // 1
         case PRICE_HIGH:     PriceType = "High";     break;  // 2
         case PRICE_LOW:      PriceType = "Low";      break;  // 3
         case PRICE_MEDIAN:   PriceType = "Median";   break;  // 4
         case PRICE_TYPICAL:  PriceType = "Typical";  break;  // 5
         case PRICE_WEIGHTED: PriceType = "Weighted"; break;  // 6
      }      

   //
   //
   //
   //
   //

   string add = ""; if (Delayed) add = "Delayed ";
   IndicatorShortName(add+"Stochastic momentum "+timeFrameToString(timeFrame)+" ("+Length+","+Smooth1+","+Smooth2+","+PriceType+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double tBuffer[][16];
#define ema10 0
#define ema11 1
#define ema12 2
#define ema20 3
#define ema21 4
#define ema22 5
#define ema30 6
#define ema31 7
#define ema32 8
#define ema40 9
#define ema41 10
#define ema42 11
#define smo0  12
#define smo1  13
#define sms0  14
#define sms1  15

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
                 if (ReturningBars)  { smo[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (CalculateValue || timeFrame == Period())
   {
      if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);
      double alpha1 = 2.0 /(1.0+Smooth1);
      double alpha2 = 2.0 /(1.0+Smooth2);
      double alphas = 2.0 /(1.0+Signal);
      
      //
      //
      //
      //
      //
      
      for(i=limit,r=Bars-i-1; i >= 0; i--,r++)
      {
         double ho = High[iHighest(NULL,0,MODE_HIGH,Length,i)];
         double lo =  Low[ iLowest(NULL,0,MODE_LOW ,Length,i)];
         double po = iMA(NULL,0,1,0,MODE_SMA,Price,i);

            if (Delayed)
            {
               
               double hh = High[iHighest(NULL,0,MODE_HIGH,Length-1,i+1)]; hh = MathMax(hh,Open[i]);
               double ll =  Low[ iLowest(NULL,0,MODE_LOW ,Length-1,i+1)]; ll = MathMin(ll,Open[i]);
               double pr = Open[i];
            }
            else
            {
               hh = ho;
               ll = lo;
               pr = po;
            }

            //
            //
            //
            //
            //
            
            tBuffer[r][ema10] = pr - 0.5*(hh+ll);
            tBuffer[r][ema20] =           hh-ll;
            tBuffer[r][ema30] = po - 0.5*(ho+lo);
            tBuffer[r][ema40] =           ho-lo;
            if (i>=Bars-3)
            {
               tBuffer[r][ema11] = tBuffer[r][ema10];
               tBuffer[r][ema12] = tBuffer[r][ema10];
               tBuffer[r][ema21] = tBuffer[r][ema20];
               tBuffer[r][ema22] = tBuffer[r][ema20];
               tBuffer[r][ema31] = tBuffer[r][ema30];
               tBuffer[r][ema32] = tBuffer[r][ema30];
               tBuffer[r][ema41] = tBuffer[r][ema40];
               tBuffer[r][ema42] = tBuffer[r][ema40];
               continue;
            }
      
         //
         //
         //
         //
         //
      
            tBuffer[r][ema11] = tBuffer[r-1][ema31] + alpha1*(tBuffer[r][ema10]-tBuffer[r-1][ema31]);
            tBuffer[r][ema12] = tBuffer[r-1][ema32] + alpha2*(tBuffer[r][ema11]-tBuffer[r-1][ema32]);
            tBuffer[r][ema21] = tBuffer[r-1][ema41] + alpha1*(tBuffer[r][ema20]-tBuffer[r-1][ema41]);
            tBuffer[r][ema22] = tBuffer[r-1][ema42] + alpha2*(tBuffer[r][ema21]-tBuffer[r-1][ema42]);
            tBuffer[r][ema31] = tBuffer[r-1][ema31] + alpha1*(tBuffer[r][ema30]-tBuffer[r-1][ema31]);
            tBuffer[r][ema32] = tBuffer[r-1][ema32] + alpha2*(tBuffer[r][ema31]-tBuffer[r-1][ema32]);
            tBuffer[r][ema41] = tBuffer[r-1][ema41] + alpha1*(tBuffer[r][ema40]-tBuffer[r-1][ema41]);
            tBuffer[r][ema42] = tBuffer[r-1][ema42] + alpha2*(tBuffer[r][ema41]-tBuffer[r-1][ema42]);
            tBuffer[r][smo0]  = 100.00 * tBuffer[r][ema12] / (0.5 * tBuffer[r][ema22]);
            tBuffer[r][smo1]  = 100.00 * tBuffer[r][ema32] / (0.5 * tBuffer[r][ema42]);
            smo[i]            = tBuffer[r][smo0];

         //
         //
         //
         //
         //
         
         if (Signal>1)
         {
            tBuffer[r][sms0] = tBuffer[r-1][sms1]+alphas*(tBuffer[r][smo0]-tBuffer[r-1][sms1]);
            tBuffer[r][sms1] = tBuffer[r-1][sms1]+alphas*(tBuffer[r][smo1]-tBuffer[r-1][sms1]);
            aup[i]           = EMPTY_VALUE;
            ado[i]           = EMPTY_VALUE;
            sms[i]           = tBuffer[r][sms0];

               state[i] = state[i+1];  
                  if (smo[i]>sms[i]) state[i] =  1;
                  if (smo[i]<sms[i]) state[i] = -1;
                  if (state[i]!=state[i+1])
                  {
                     if (state[i] ==  1) aup[i] = sms[i];
                     if (state[i] == -1) ado[i] = sms[i];
                  }
         }
      }
      return(0);
   }
   
   //
   //
   //
   //
   //
    
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i >= 0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         smo[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateSmi",Length,Smooth1,Smooth2,Signal,Price,Delayed,0,y);
         
         //
         //
         //
         //
         //
         
            if (Signal>1)
            {
               aup[i] = EMPTY_VALUE;
               ado[i] = EMPTY_VALUE;
               sms[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateSmi",Length,Smooth1,Smooth2,Signal,Price,Delayed,1,y);

               state[i] = state[i+1];  
                  if (smo[i]>sms[i]) state[i] =  1;
                  if (smo[i]<sms[i]) state[i] = -1;
                  if (state[i]!=state[i+1])
                  {
                     if (state[i] ==  1) aup[i] = sms[i];
                     if (state[i] == -1) ado[i] = sms[i];
                  }
            }
            if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
            {
               smo[i+k] = k*factor*smo[i+n] + (1.0-k*factor)*smo[i];
               if (Signal>1)
               {
                  sms[i+k] = k*factor*sms[i+n] + (1.0-k*factor)*sms[i];
                  if (aup[i+k] != EMPTY_VALUE) aup[i+k]=sms[i+k];
                  if (ado[i+k] != EMPTY_VALUE) ado[i+k]=sms[i+k];
               }
            }
   }
   
   //
   //
   //
   //
   //
   
   return(0);
         
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}